@@ -24,11 +24,15 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#if defined (__linux__)
+
+#if defined(STRESS_IOPRIO)
+
 #include <sys/syscall.h>
 
 /*
@@ -47,7 +51,6 @@
 #define IOPRIO_PRIO_VALUE(class, data)	(((class) << 13) | data)
 #endif
 
-#include "stress-ng.h"
 
 /*
  *  get_opt_ionice_class()
@@ -55,16 +58,16 @@
  */
 int get_opt_ionice_class(const char *const str)
 {
-#if defined (IOPRIO_CLASS_IDLE)
+#if defined(IOPRIO_CLASS_IDLE)
 	if (!strcmp("idle", str))
 		return IOPRIO_CLASS_IDLE;
 #endif
-#if defined (IOPRIO_CLASS_BE)
+#if defined(IOPRIO_CLASS_BE)
 	if (!strcmp("besteffort", str) ||
 	    !strcmp("be", str))
 		return IOPRIO_CLASS_BE;
 #endif
-#if defined (IOPRIO_CLASS_RT)
+#if defined(IOPRIO_CLASS_RT)
 	if (!strcmp("realtime", str) ||
 	    !strcmp("rt", str))
 		return IOPRIO_CLASS_RT;
@@ -73,13 +76,13 @@ int get_opt_ionice_class(const char *const str)
 		fprintf(stderr, "Invalid ionice-class option: %s\n", str);
 
 	fprintf(stderr, "Available options are:");
-#if defined (IOPRIO_CLASS_IDLE)
+#if defined(IOPRIO_CLASS_IDLE)
 	fprintf(stderr, " idle");
 #endif
-#if defined (IOPRIO_CLASS_BE)
+#if defined(IOPRIO_CLASS_BE)
 	fprintf(stderr, " besteffort be");
 #endif
-#if defined (IOPRIO_CLASS_RT)
+#if defined(IOPRIO_CLASS_RT)
 	fprintf(stderr, " realtime rt");
 #endif
 	fprintf(stderr, "\n");
@@ -87,7 +90,7 @@ int get_opt_ionice_class(const char *const str)
 	exit(EXIT_FAILURE);
 }
 
-#if defined (__linux__)
+#if defined(STRESS_IOPRIO)
 /*
  *  ioprio_set()
  *	ioprio_set system call
