@@ -28,6 +28,7 @@
 #include <stdint.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <syslog.h>
 
 #include "stress-ng.h"
 
@@ -40,7 +41,7 @@ bool	 abort_msg_emitted;
  */
 int print(
 	FILE *fp,
-	const int flag,
+	const uint64_t flag,
 	const char *const fmt, ...)
 {
 	va_list ap;
@@ -58,14 +59,12 @@ int print(
 			type = "debug";
 		if (flag & PR_INFO)
 			type = "info";
-		if (flag & PR_FAIL) {
+		if (flag & PR_FAIL)
 			type = "fail";
-		}
 
-		n = snprintf(buf, sizeof(buf), "%s: %s: [%i] ",
-			app_name, type, getpid());
+		n = snprintf(buf, sizeof(buf), "%s: [%i] ", type, getpid());
 		ret = vsnprintf(buf + n, sizeof(buf) - n, fmt, ap);
-		fprintf(fp, "%s", buf);
+		fprintf(fp, "%s: %s", app_name, buf);
 		fflush(fp);
 
 		if (flag & PR_FAIL) {
@@ -79,6 +78,12 @@ int print(
 				}
 			}
 		}
+
+		/* Log messages if syslog requested, don't log DEBUG */
+		if ((opt_flags & OPT_FLAGS_SYSLOG) &&
+		    (!(flag & PR_DEBUG))) {
+			syslog(LOG_INFO, "%s", buf);
+		}
 	}
 	va_end(ap);
 
@@ -89,9 +94,13 @@ int print(
  *  pr_failed()
  *	print failure message with errno
  */
-void pr_failed(const int flag, const char *name, const char *what)
+void pr_failed(
+	const uint64_t flag,
+	const char *name,
+	const char *what,
+	const int err)
 {
 	print(stderr, flag, "%s: %s failed, errno=%d (%s)\n",
-		name, what, errno, strerror(errno));
+		name, what, err, strerror(err));
 }
 
