@@ -22,9 +22,11 @@
  * functionality.
  *
  */
-#define _GNU_SOURCE 
+#define _GNU_SOURCE
 
-#if defined(__linux__)
+#include "stress-ng.h"
+
+#if defined(STRESS_AFFINITY)
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,7 +34,6 @@
 #include <unistd.h>
 #include <sched.h>
 
-#include "stress-ng.h"
 
 /*
  *  stress on sched_affinity()
@@ -57,7 +58,8 @@ int stress_affinity(
 		CPU_ZERO(&mask);
 		CPU_SET(cpu, &mask);
 		if (sched_setaffinity(0, sizeof(mask), &mask) < 0) {
-			pr_fail(stderr, "failed to move to CPU %lu\n", cpu);
+			pr_fail(stderr, "%s: failed to move to CPU %lu\n",
+				name, cpu);
 		} else {
 			/* Now get and check */
 			CPU_ZERO(&mask);
@@ -65,7 +67,8 @@ int stress_affinity(
 			sched_getaffinity(0, sizeof(mask), &mask);
 			if ((opt_flags & OPT_FLAGS_VERIFY) &&
 			    (!CPU_ISSET(cpu, &mask)))
-				pr_fail(stderr, "failed to move to CPU %lu\n", cpu);
+				pr_fail(stderr, "%s: failed to move to CPU %lu\n",
+					name, cpu);
 		}
 		(*counter)++;
 	} while (opt_do_run && (!max_ops || *counter < max_ops));
