@@ -24,7 +24,9 @@
  */
 #define _GNU_SOURCE
 
-#if defined(__linux__)
+#include "stress-ng.h"
+
+#if defined(STRESS_EPOLL)
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -53,8 +55,6 @@
 #include <sched.h>
 #endif
 
-#include "stress-ng.h"
-
 #define MAX_EPOLL_EVENTS 	(1024)
 #define MAX_SERVERS		(4)
 
@@ -334,7 +334,7 @@ retry:
 			return -1;
 		}
 		errno = 0;
-	
+
 		stress_set_sockaddr(name, instance, ppid,
 			opt_epoll_domain, port, &addr, &addr_len);
 
