@@ -24,6 +24,10 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
+#if defined(STRESS_FALLOCATE)
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -34,9 +38,32 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "stress-ng.h"
+static off_t opt_fallocate_bytes = DEFAULT_FALLOCATE_BYTES;
+static bool set_fallocate_bytes = false;
+
+void stress_set_fallocate_bytes(const char *optarg)
+{
+	set_fallocate_bytes = true;
+	opt_fallocate_bytes = (off_t)get_uint64_byte(optarg);
+	check_range("fallocate-bytes", opt_fallocate_bytes,
+		MIN_FALLOCATE_BYTES, MAX_FALLOCATE_BYTES);
+}
+
+#if defined(__linux__)
+static const int modes[] = {
+#if defined(FALLOC_FL_KEEP_SIZE)
+	FALLOC_FL_KEEP_SIZE,
+#endif
+#if defined(FALLOC_FL_KEEP_SIZE) && defined(FALLOC_FL_PUNCH_HOLE)
+	FALLOC_FL_KEEP_SIZE | FALLOC_FL_PUNCH_HOLE,
+#endif
+#if defined(FALLOC_FL_ZERO_RANGE)
+	FALLOC_FL_ZERO_RANGE,
+#endif
+	0
+};
+#endif
 
-#if _XOPEN_SOURCE >= 600 || _POSIX_C_SOURCE >= 200112L
 /*
  *  stress_fallocate
  *	stress I/O via fallocate and ftruncate
@@ -52,6 +79,13 @@ int stress_fallocate(
 	char filename[PATH_MAX];
 	uint64_t ftrunc_errs = 0;
 
+	if (!set_fallocate_bytes) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_fallocate_bytes = MAX_FALLOCATE_BYTES;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_fallocate_bytes = MIN_FALLOCATE_BYTES;
+	}
+
 	if (stress_temp_dir_mk(name, pid, instance) < 0)
 		return EXIT_FAILURE;
 
@@ -66,32 +100,69 @@ int stress_fallocate(
 	(void)unlink(filename);
 
 	do {
-		(void)posix_fallocate(fd, (off_t)0, 4096 * 4096);
+		(void)posix_fallocate(fd, (off_t)0, opt_fallocate_bytes);
 		if (!opt_do_run)
 			break;
-		fsync(fd);
+		(void)fsync(fd);
 		if (opt_flags & OPT_FLAGS_VERIFY) {
 			struct stat buf;
 
 			if (fstat(fd, &buf) < 0)
-				pr_fail(stderr, "fstat on file failed");
-			else if (buf.st_size != (off_t)4096 * 4096)
-					pr_fail(stderr, "file size does not match size the expected file size\n");
+				pr_fail(stderr, "%s: fstat on file failed", name);
+			else if (buf.st_size != opt_fallocate_bytes)
+				pr_fail(stderr, "%s: file size %jd does not match size the expected file size of %jd\n",
+					name, (intmax_t)buf.st_size, (intmax_t)opt_fallocate_bytes);
 		}
 
 		if (ftruncate(fd, 0) < 0)
 			ftrunc_errs++;
 		if (!opt_do_run)
 			break;
-		fsync(fd);
+		(void)fsync(fd);
+
 		if (opt_flags & OPT_FLAGS_VERIFY) {
 			struct stat buf;
 
 			if (fstat(fd, &buf) < 0)
-				pr_fail(stderr, "fstat on file failed");
+				pr_fail(stderr, "%s: fstat on file failed", name);
 			else if (buf.st_size != (off_t)0)
-					pr_fail(stderr, "file size does not match size the expected file size\n");
+				pr_fail(stderr, "%s: file size %jd does not match size the expected file size of 0\n",
+					name, (intmax_t)buf.st_size);
+
 		}
+
+		if (ftruncate(fd, opt_fallocate_bytes) < 0)
+			ftrunc_errs++;
+		(void)fsync(fd);
+		if (ftruncate(fd, 0) < 0)
+			ftrunc_errs++;
+		(void)fsync(fd);
+
+#if defined(__linux__)
+		if (SIZEOF_ARRAY(modes) > 1) {
+			/*
+			 *  non-portable Linux fallocate()
+			 */
+			int i;
+			(void)fallocate(fd, 0, (off_t)0, opt_fallocate_bytes);
+			if (!opt_do_run)
+				break;
+			(void)fsync(fd);
+
+			for (i = 0; i < 64; i++) {
+				off_t offset = (mwc() % opt_fallocate_bytes) & ~0xfff;
+				int j = (mwc() >> 8) % SIZEOF_ARRAY(modes);
+
+				(void)fallocate(fd, modes[j], offset, 64 * KB);
+				if (!opt_do_run)
+					break;
+				(void)fsync(fd);
+			}
+			if (ftruncate(fd, 0) < 0)
+				ftrunc_errs++;
+			(void)fsync(fd);
+		}
+#endif
 		(*counter)++;
 	} while (opt_do_run && (!max_ops || *counter < max_ops));
 	if (ftrunc_errs)
