@@ -80,7 +80,10 @@ int stress_fault(
 		{
 			char buffer[1];
 
-			if (write(fd, buffer, sizeof(buffer)) < 0) {
+redo:
+			if (opt_do_run && (write(fd, buffer, sizeof(buffer)) < 0)) {
+				if ((errno == EAGAIN) || (errno == EINTR))
+					goto redo;
 				(void)close(fd);
 				pr_err(stderr, "%s: write failed: errno=%d (%s)\n",
 					name, errno, strerror(errno));
@@ -126,8 +129,8 @@ int stress_fault(
 	(void)stress_temp_dir_rm(name, pid, instance);
 
 	if (!getrusage(RUSAGE_SELF, &usage)) {
-		pr_dbg(stderr, "page faults: minor: %lu, major: %lu\n",
-			usage.ru_minflt, usage.ru_majflt);
+		pr_dbg(stderr, "%s: page faults: minor: %lu, major: %lu\n",
+			name, usage.ru_minflt, usage.ru_majflt);
 	}
 
 	return EXIT_SUCCESS;
