@@ -34,7 +34,11 @@
 #include "stress-ng.h"
 
 static uint64_t opt_fork_max = DEFAULT_FORKS;
+static bool set_fork_max = false;
+#if defined(STRESS_VFORK)
 static uint64_t opt_vfork_max = DEFAULT_VFORKS;
+static bool set_vfork_max = false;
+#endif
 
 /*
  *  stress_set_fork_max()
@@ -42,21 +46,25 @@ static uint64_t opt_vfork_max = DEFAULT_VFORKS;
  */
 void stress_set_fork_max(const char *optarg)
 {
+	set_fork_max = true;
 	opt_fork_max = get_uint64_byte(optarg);
 	check_range("fork-max", opt_fork_max,
 		MIN_FORKS, MAX_FORKS);
 }
 
+#if defined(STRESS_VFORK)
 /*
  *  stress_set_vfork_max()
  *	set maximum number of vforks allowed
  */
 void stress_set_vfork_max(const char *optarg)
 {
+	set_vfork_max = true;
 	opt_vfork_max = get_uint64_byte(optarg);
 	check_range("vfork-max", opt_vfork_max,
 		MIN_VFORKS, MAX_VFORKS);
 }
+#endif
 
 /*
  *  stress_fork_fn()
@@ -95,14 +103,14 @@ int stress_fork_fn(
 			if (pids[i] > 0) {
 				int status;
 				/* Parent, wait for child */
-				waitpid(pids[i], &status, 0);
+				(void)waitpid(pids[i], &status, 0);
 				(*counter)++;
 			}
 		}
 
 		for (i = 0; i < fork_max; i++) {
 			if ((pids[i] < 0) && (opt_flags & OPT_FLAGS_VERIFY)) {
-				pr_fail(stderr, "fork failed\n");
+				pr_fail(stderr, "%s: fork failed\n", name);
 			}
 		}
 	} while (opt_do_run && (!max_ops || *counter < max_ops));
@@ -120,14 +128,19 @@ int stress_fork(
 	const uint64_t max_ops,
 	const char *name)
 {
+	if (!set_fork_max) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_fork_max = MAX_FORKS;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_fork_max = MIN_FORKS;
+	}
+
 	return stress_fork_fn(counter, instance, max_ops,
 		name, fork, opt_fork_max);
 }
 
 
-#if _BSD_SOURCE || \
-   (_XOPEN_SOURCE >= 500 || _XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED) && \
-   !(_POSIX_C_SOURCE >= 200809L || _XOPEN_SOURCE >= 700)
+#if defined(STRESS_VFORK)
 /*
  *  stress_vfork()
  *	stress by vforking and exiting
@@ -138,6 +151,13 @@ int stress_vfork(
 	const uint64_t max_ops,
 	const char *name)
 {
+	if (!set_vfork_max) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_vfork_max = MAX_VFORKS;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_vfork_max = MIN_VFORKS;
+	}
+
 	return stress_fork_fn(counter, instance, max_ops,
 		name, vfork, opt_vfork_max);
 }
