@@ -32,6 +32,7 @@
 #include "stress-ng.h"
 
 static uint64_t opt_hsearch_size = DEFAULT_HSEARCH_SIZE;
+static bool set_hsearch_size = false;
 
 /*
  *  stress_set_hsearch_size()
@@ -39,6 +40,7 @@ static uint64_t opt_hsearch_size = DEFAULT_HSEARCH_SIZE;
  */
 void stress_set_hsearch_size(const char *optarg)
 {
+	set_hsearch_size = true;
         opt_hsearch_size = get_uint64_byte(optarg);
         check_range("hsearch-size", opt_hsearch_size,
                 MIN_TSEARCH_SIZE, MAX_TSEARCH_SIZE);
@@ -54,13 +56,21 @@ int stress_hsearch(
 	const uint64_t max_ops,
 	const char *name)
 {
-	const size_t max = (size_t)opt_hsearch_size;
-	size_t i;
+	size_t i, max = (size_t)opt_hsearch_size;
 	int ret = EXIT_FAILURE;
 	char **keys;
 
 	(void)instance;
 
+	if (!set_hsearch_size) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_hsearch_size = MAX_HSEARCH_SIZE;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_hsearch_size = MIN_HSEARCH_SIZE;
+	}
+
+	max = (size_t)opt_hsearch_size;
+
 	/* Make hash table with 25% slack */
 	if (!hcreate(max + (max / 4))) {
 		pr_failed_err(name, "hcreate");
@@ -102,10 +112,10 @@ int stress_hsearch(
 			ep = hsearch(e, FIND);
 			if (opt_flags & OPT_FLAGS_VERIFY) {
 				if (ep == NULL) {
-					pr_fail(stderr, "cannot find key %s\n", keys[i]);
+					pr_fail(stderr, "%s: cannot find key %s\n", name, keys[i]);
 				} else {
 					if (i != (size_t)ep->data) {
-						pr_fail(stderr, "hash returned incorrect data %zd\n", i);
+						pr_fail(stderr, "%s: hash returned incorrect data %zd\n", name, i);
 					}
 				}
 			}
