@@ -26,7 +26,9 @@
  */
 #define _GNU_SOURCE
 
-#if defined(__linux__)
+#include "stress-ng.h"
+
+#if defined(STRESS_INOTIFY)
 
 #include <stdio.h>
 #include <stdint.h>
@@ -44,7 +46,6 @@
 #include <sys/select.h>
 #include <sys/inotify.h>
 
-#include "stress-ng.h"
 
 #define DIR_FLAGS	(S_IRWXU | S_IRWXG)
 #define FILE_FLAGS	(S_IRUSR | S_IWUSR)
@@ -52,8 +53,8 @@
 #define TIME_OUT	(10)	/* Secs for inotify to report back */
 #define BUF_SIZE	(4096)
 
-typedef int (*inotify_helper)(const char *path, const void *private);
-typedef void (*inotify_func)(const char *path);
+typedef int (*inotify_helper)(const char *name, const char *path, const void *private);
+typedef void (*inotify_func)(const char *name, const char *path);
 
 typedef struct {
 	const inotify_func func;
@@ -66,6 +67,7 @@ typedef struct {
  *	required inotify event flags 'flags'.
  */
 static void inotify_exercise(
+	const char *name,	/* Stressor name */
 	const char *filename,	/* Filename in test */
 	const char *watchname,	/* File or directory to watch using inotify */
 	const char *matchname,	/* Filename we expect inotify event to report */
@@ -91,19 +93,19 @@ retry:
 			goto retry;
 		}
 		/* Nope, give up */
-		pr_fail(stderr, "inotify_init failed: errno=%d (%s) after %" PRIu32 " calls\n",
-			errno, strerror(errno), n);
+		pr_fail(stderr, "%s: inotify_init failed: errno=%d (%s) after %" PRIu32 " calls\n",
+			name, errno, strerror(errno), n);
 		return;
 	}
 
 	if ((wd = inotify_add_watch(fd, watchname, flags)) < 0) {
 		(void)close(fd);
-		pr_fail(stderr, "inotify_add_watch failed: errno=%d (%s)",
-			errno, strerror(errno));
+		pr_fail(stderr, "%s: inotify_add_watch failed: errno=%d (%s)",
+			name, errno, strerror(errno));
 		return;
 	}
 
-	if (func(filename, private) < 0)
+	if (func(name, filename, private) < 0)
 		goto cleanup;
 
 	while (check_flags) {
@@ -123,25 +125,32 @@ retry:
 		err = select(fd + 1, &rfds, NULL, NULL, &tv);
 		if (err == -1) {
 			if (errno != EINTR)
-				pr_err(stderr, "select error: errno=%d (%s)\n",
-					errno, strerror(errno));
+				pr_err(stderr, "%s: select error: errno=%d (%s)\n",
+					name, errno, strerror(errno));
 			break;
 		} else if (err == 0) {
 			if (opt_flags & OPT_FLAGS_VERIFY)
-				pr_fail(stderr, "timed waiting for event flags 0x%x\n", flags);
+				pr_fail(stderr, "%s: timed waiting for event flags 0x%x\n", name, flags);
 			break;
 		}
 
+redo:
+		if (!opt_do_run)
+			break;
 		len = read(fd, buffer, sizeof(buffer));
 
-		if ((len < 0) || (len > (ssize_t)sizeof(buffer))) {
-			pr_fail(stderr, "error reading inotify: errno=%d (%s)\n",
-				errno, strerror(errno));
-			break;
+		if ((len <= 0) || (len > (ssize_t)sizeof(buffer))) {
+			if ((errno == EAGAIN) || (errno == EINTR))
+				goto redo;
+			if (errno) {
+				pr_fail(stderr, "%s: error reading inotify: errno=%d (%s)\n",
+					name, errno, strerror(errno));
+				break;
+			}
 		}
 
 		/* Scan through inotify events */
-		do {
+		while ((i >= 0) && (i <= len - (ssize_t)sizeof(struct inotify_event))) {
 			struct inotify_event *event = (struct inotify_event *)&buffer[i];
 			int f = event->mask & (IN_DELETE_SELF | IN_MOVE_SELF |
 					       IN_MOVED_TO | IN_MOVED_FROM |
@@ -154,14 +163,14 @@ retry:
 				check_flags &= ~(flags & event->mask);
 
 			i += sizeof(struct inotify_event) + event->len;
-		} while (i < len);
+		}
 	}
 
 cleanup:
 	(void)inotify_rm_watch(fd, wd);
 	if (close(fd) < 0) {
-		pr_err(stderr, "close error: errno=%d (%s)\n",
-			errno, strerror(errno));
+		pr_err(stderr, "%s: close error: errno=%d (%s)\n",
+			name, errno, strerror(errno));
 	}
 }
 
@@ -169,11 +178,11 @@ cleanup:
  *  rm_file()
  *	remove a file
  */
-static int rm_file(const char *path)
+static int rm_file(const char *name, const char *path)
 {
 	if ((unlink(path) < 0) && errno != ENOENT) {
-		pr_err(stderr, "cannot remove file %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot remove file %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		return -1;
 	}
 	return 0;
@@ -183,7 +192,7 @@ static int rm_file(const char *path)
  *  rm_dir()
  *	clean files in directory and directory
  */
-static int rm_dir(const char *path)
+static int rm_dir(const char *name, const char *path)
 {
 	DIR *dp;
 	int ret;
@@ -200,14 +209,14 @@ static int rm_dir(const char *path)
 				continue;
 
 			snprintf(filename, sizeof(filename), "%s/%s", path, d->d_name);
-			(void)rm_file(filename);
+			(void)rm_file(name, filename);
 		}
 		(void)closedir(dp);
 	}
 	ret = rmdir(path);
 	if (ret < 0 && errno != ENOENT)
-		pr_err(stderr, "cannot remove directory %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot remove directory %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 	return ret;
 }
 
@@ -215,11 +224,11 @@ static int rm_dir(const char *path)
  *  mk_dir()
  *	make a directory
  */
-static int mk_dir(const char *path)
+static int mk_dir(const char *name, const char *path)
 {
 	if (mkdir(path, DIR_FLAGS) < 0) {
-		pr_err(stderr, "cannot mkdir %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot mkdir %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		return -1;
 	}
 	return 0;
@@ -242,17 +251,17 @@ static void mk_filename(
  *  mk_file()
  *	create file of length len bytes
  */
-static int mk_file(const char *filename, const size_t len)
+static int mk_file(const char *name, const char *filename, const size_t len)
 {
 	int fd;
 	size_t sz = len;
 
 	char buffer[BUF_SIZE];
 
-	(void)rm_file(filename);
+	(void)rm_file(name, filename);
 	if ((fd = open(filename, O_CREAT | O_RDWR, FILE_FLAGS)) < 0) {
-		pr_err(stderr, "cannot create file %s: errno=%d (%s)\n",
-			filename, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot create file %s: errno=%d (%s)\n",
+			name, filename, errno, strerror(errno));
 		return -1;
 	}
 
@@ -262,8 +271,8 @@ static int mk_file(const char *filename, const size_t len)
 		int ret;
 
 		if ((ret = write(fd, buffer, n)) < 0) {
-			pr_err(stderr, "error writing to file %s: errno=%d (%s)\n",
-				filename, errno, strerror(errno));
+			pr_err(stderr, "%s: error writing to file %s: errno=%d (%s)\n",
+				name, filename, errno, strerror(errno));
 			(void)close(fd);
 			return -1;
 		}
@@ -271,38 +280,38 @@ static int mk_file(const char *filename, const size_t len)
 	}
 
 	if (close(fd) < 0) {
-		pr_err(stderr, "cannot close file %s: errno=%d (%s)\n",
-			filename, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot close file %s: errno=%d (%s)\n",
+			name, filename, errno, strerror(errno));
 		return -1;
 	}
 	return 0;
 }
 
 
-static int inotify_attrib_helper(const char *path, const void *dummy)
+static int inotify_attrib_helper(const char *name, const char *path, const void *dummy)
 {
 	(void)dummy;
 	if (chmod(path, S_IRUSR | S_IWUSR) < 0) {
-		pr_err(stderr, "cannot chmod file %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot chmod file %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		return -1;
 	}
 	return 0;
 }
 
-void inotify_attrib_file(const char *path)
+void inotify_attrib_file(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_file");
-	if (mk_file(filepath, 4096) < 0)
+	if (mk_file(name, filepath, 4096) < 0)
 		return;
 
-	inotify_exercise(filepath, path, "inotify_file", inotify_attrib_helper, IN_ATTRIB, NULL);
-	(void)rm_file(filepath);
+	inotify_exercise(name, filepath, path, "inotify_file", inotify_attrib_helper, IN_ATTRIB, NULL);
+	(void)rm_file(name, filepath);
 }
 
-static int inotify_access_helper(const char *path, const void *dummy)
+static int inotify_access_helper(const char *name, const char *path, const void *dummy)
 {
 	int fd;
 	char buffer[1];
@@ -310,289 +319,299 @@ static int inotify_access_helper(const char *path, const void *dummy)
 
 	(void)dummy;
 	if ((fd = open(path, O_RDONLY)) < 0) {
-		pr_err(stderr, "cannot open file %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot open file %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		return -1;
 	}
 
 	/* Just want to force an access */
-	if (read(fd, buffer, 1) < 0) {
-		pr_err(stderr, "cannot read file %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+do_access:
+	if (opt_do_run && (read(fd, buffer, 1) < 0)) {
+		if ((errno == EAGAIN) || (errno == EINTR))
+			goto do_access;
+		pr_err(stderr, "%s: cannot read file %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		rc = -1;
 	}
 	(void)close(fd);
 	return rc;
 }
 
-static void inotify_access_file(const char *path)
+static void inotify_access_file(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_file");
-	if (mk_file(filepath, 4096) < 0)
+	if (mk_file(name, filepath, 4096) < 0)
 		return;
 
-	inotify_exercise(filepath, path, "inotify_file", inotify_access_helper, IN_ACCESS, NULL);
-	(void)rm_file(filepath);
+	inotify_exercise(name, filepath, path, "inotify_file", inotify_access_helper, IN_ACCESS, NULL);
+	(void)rm_file(name, filepath);
 }
 
-static int inotify_modify_helper(const char *path, const void *dummy)
+static int inotify_modify_helper(const char *name, const char *path, const void *dummy)
 {
 	int fd, rc = 0;
 	char buffer[1] = { 0 };
 
 	(void)dummy;
-	if (mk_file(path, 4096) < 0)
+	if (mk_file(name, path, 4096) < 0)
 		return -1;
 	if ((fd = open(path, O_RDWR)) < 0) {
-		pr_err(stderr, "cannot open file %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot open file %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		rc = -1;
 		goto remove;
 	}
-	if (write(fd, buffer, 1) < 0) {
-		pr_err(stderr, "cannot write to file %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+do_modify:
+	if (opt_do_run && (write(fd, buffer, 1) < 0)) {
+		if ((errno == EAGAIN) || (errno == EINTR))
+			goto do_modify;
+		pr_err(stderr, "%s: cannot write to file %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		rc = -1;
 	}
 	(void)close(fd);
 remove:
-	(void)rm_file(path);
+	(void)rm_file(name, path);
 	return rc;
 }
 
-static void inotify_modify_file(const char *path)
+static void inotify_modify_file(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_file");
-	inotify_exercise(filepath, path, "inotify_file", inotify_modify_helper, IN_MODIFY, NULL);
+	inotify_exercise(name, filepath, path, "inotify_file", inotify_modify_helper, IN_MODIFY, NULL);
 }
 
-static int inotify_creat_helper(const char *path, const void *dummy)
+static int inotify_creat_helper(const char *name, const char *path, const void *dummy)
 {
 	(void)dummy;
 	int fd;
 	if ((fd = creat(path, FILE_FLAGS)) < 0) {
-		pr_err(stderr, "cannot create file %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot create file %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		return -1;
 	}
 	(void)close(fd);
 	return 0;
 }
 
-static void inotify_creat_file(const char *path)
+static void inotify_creat_file(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_file");
-	inotify_exercise(filepath, path, "inotify_file", inotify_creat_helper, IN_CREATE, NULL);
-	(void)rm_file(filepath);
+	inotify_exercise(name, filepath, path, "inotify_file", inotify_creat_helper, IN_CREATE, NULL);
+	(void)rm_file(name, filepath);
 }
 
-static int inotify_open_helper(const char *path, const void *dummy)
+static int inotify_open_helper(const char *name, const char *path, const void *dummy)
 {
 	int fd;
 
 	(void)dummy;
 	if ((fd = open(path, O_RDONLY)) < 0) {
-		pr_err(stderr, "cannot open file %s: errno=%d (%s)\n",
-			path, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot open file %s: errno=%d (%s)\n",
+			name, path, errno, strerror(errno));
 		return -1;
 	}
 	(void)close(fd);
 	return 0;
 }
 
-static void inotify_open_file(const char *path)
+static void inotify_open_file(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_file");
-	if (mk_file(filepath, 4096) < 0)
+	if (mk_file(name, filepath, 4096) < 0)
 		return;
-	inotify_exercise(filepath, path, "inotify_file", inotify_open_helper, IN_OPEN, NULL);
-	(void)rm_file(filepath);
+	inotify_exercise(name, filepath, path, "inotify_file", inotify_open_helper, IN_OPEN, NULL);
+	(void)rm_file(name, filepath);
 }
 
-static int inotify_delete_helper(const char *path, const void *dummy)
+static int inotify_delete_helper(const char *name, const char *path, const void *dummy)
 {
 	(void)dummy;
 
-	return rm_file(path);
+	return rm_file(name, path);
 }
 
-static void inotify_delete_file(const char *path)
+static void inotify_delete_file(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_file");
-	if (mk_file(filepath, 4096) < 0)
+	if (mk_file(name, filepath, 4096) < 0)
 		return;
-	inotify_exercise(filepath, path, "inotify_file", inotify_delete_helper, IN_DELETE, NULL);
+	inotify_exercise(name, filepath, path, "inotify_file", inotify_delete_helper, IN_DELETE, NULL);
 	/* We remove (again) it just in case the test failed */
-	(void)rm_file(filepath);
+	(void)rm_file(name, filepath);
 }
 
-static int inotify_delete_self_helper(const char *path, const void *dummy)
+static int inotify_delete_self_helper(const char *name, const char *path, const void *dummy)
 {
 	(void)dummy;
 
-	return rm_dir(path);
+	return rm_dir(name, path);
 }
 
-static void inotify_delete_self(const char *path)
+static void inotify_delete_self(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_dir");
-	if (mk_dir(filepath) < 0)
+	if (mk_dir(name, filepath) < 0)
 		return;
-	inotify_exercise(filepath, filepath, "inotify_dir", inotify_delete_self_helper, IN_DELETE_SELF, NULL);
+	inotify_exercise(name, filepath, filepath, "inotify_dir", inotify_delete_self_helper, IN_DELETE_SELF, NULL);
 	/* We remove (again) in case the test failed */
-	(void)rm_dir(filepath);
+	(void)rm_dir(name, filepath);
 }
 
-static int inotify_move_self_helper(const char *oldpath, const void *private)
+static int inotify_move_self_helper(const char *name, const char *oldpath, const void *private)
 {
 	char *newpath = (char*)private;
 
 	if (rename(oldpath, newpath) < 0) {
-		pr_err(stderr, "cannot rename %s to %s: errno=%d (%s)\n",
-			oldpath, newpath, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot rename %s to %s: errno=%d (%s)\n",
+			name, oldpath, newpath, errno, strerror(errno));
 		return -1;
 	}
 	return 0;
 }
 
-static void inotify_move_self(const char *path)
+static void inotify_move_self(const char *name, const char *path)
 {
 	char filepath[PATH_MAX], newpath[PATH_MAX];
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_dir");
-	if (mk_dir(filepath) < 0)
+	if (mk_dir(name, filepath) < 0)
 		return;
 	mk_filename(newpath, PATH_MAX, path, "renamed_dir");
 
-	inotify_exercise(filepath, filepath, "inotify_dir", inotify_move_self_helper, IN_MOVE_SELF, newpath);
-	(void)rm_dir(newpath);
-	(void)rm_dir(filepath);	/* In case rename failed */
+	inotify_exercise(name, filepath, filepath, "inotify_dir", inotify_move_self_helper, IN_MOVE_SELF, newpath);
+	(void)rm_dir(name, newpath);
+	(void)rm_dir(name, filepath);	/* In case rename failed */
 }
 
-static int inotify_moved_to_helper(const char *newpath, const void *private)
+static int inotify_moved_to_helper(const char *name, const char *newpath, const void *private)
 {
 	char *oldpath = (char*)private;
 
 	if (rename(oldpath, newpath) < 0) {
-		pr_err(stderr, "cannot rename %s to %s: errno=%d (%s)\n",
-			oldpath, newpath, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot rename %s to %s: errno=%d (%s)\n",
+			name, oldpath, newpath, errno, strerror(errno));
 		return -1;
 	}
 	return 0;
 }
 
-static void inotify_moved_to(const char *path)
+static void inotify_moved_to(const char *name, const char *path)
 {
 	char olddir[PATH_MAX], oldfile[PATH_MAX], newfile[PATH_MAX];
 
 	mk_filename(olddir, PATH_MAX, path, "new_dir");
-	(void)rm_dir(olddir);
-	if (mk_dir(olddir) < 0)
+	(void)rm_dir(name, olddir);
+	if (mk_dir(name, olddir) < 0)
 		return;
 	mk_filename(oldfile, PATH_MAX, olddir, "inotify_file");
-	if (mk_file(oldfile, 4096) < 0)
+	if (mk_file(name, oldfile, 4096) < 0)
 		return;
 
 	mk_filename(newfile, PATH_MAX, path, "inotify_file");
-	inotify_exercise(newfile, path, "inotify_dir", inotify_moved_to_helper, IN_MOVED_TO, oldfile);
-	(void)rm_file(newfile);
-	(void)rm_dir(olddir);
+	inotify_exercise(name, newfile, path, "inotify_dir", inotify_moved_to_helper, IN_MOVED_TO, oldfile);
+	(void)rm_file(name, newfile);
+	(void)rm_dir(name, olddir);
 }
 
-static int inotify_moved_from_helper(const char *oldpath, const void *private)
+static int inotify_moved_from_helper(const char *name, const char *oldpath, const void *private)
 {
 	char *newpath = (char*)private;
 
 	if (rename(oldpath, newpath) < 0) {
-		pr_err(stderr, "cannot rename %s to %s: errno=%d (%s)\n",
-			oldpath, newpath, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot rename %s to %s: errno=%d (%s)\n",
+			name, oldpath, newpath, errno, strerror(errno));
 		return -1;
 	}
 	return 0;
 }
 
-static void inotify_moved_from(const char *path)
+static void inotify_moved_from(const char *name, const char *path)
 {
 	char oldfile[PATH_MAX], newdir[PATH_MAX], newfile[PATH_MAX];
 
 	mk_filename(oldfile, PATH_MAX, path, "inotify_file");
-	if (mk_file(oldfile, 4096) < 0)
+	if (mk_file(name, oldfile, 4096) < 0)
 		return;
 	mk_filename(newdir, PATH_MAX, path, "new_dir");
-	(void)rm_dir(newdir);
-	if (mk_dir(newdir) < 0)
+	(void)rm_dir(name, newdir);
+	if (mk_dir(name, newdir) < 0)
 		return;
 	mk_filename(newfile, PATH_MAX, newdir, "inotify_file");
-	inotify_exercise(oldfile, path, "inotify_dir", inotify_moved_from_helper, IN_MOVED_FROM, newfile);
-	(void)rm_file(newfile);
-	(void)rm_file(oldfile);	/* In case rename failed */
-	(void)rm_dir(newdir);
+	inotify_exercise(name, oldfile, path, "inotify_dir", inotify_moved_from_helper, IN_MOVED_FROM, newfile);
+	(void)rm_file(name, newfile);
+	(void)rm_file(name, oldfile);	/* In case rename failed */
+	(void)rm_dir(name, newdir);
 }
 
-static int inotify_close_write_helper(const char *path, const void *fdptr)
+static int inotify_close_write_helper(const char *name, const char *path, const void *fdptr)
 {
+	(void)name;
 	(void)path;
+
 	(void)close(*(int *)fdptr);
 	return 0;
 }
 
-static void inotify_close_write_file(const char *path)
+static void inotify_close_write_file(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 	int fd;
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_file");
-	if (mk_file(filepath, 4096) < 0)
+	if (mk_file(name, filepath, 4096) < 0)
 		return;
 
 	if ((fd = open(filepath, O_RDWR)) < 0) {
-		pr_err(stderr, "cannot re-open %s: errno=%d (%s)\n",
-			filepath, errno, strerror(errno));
+		pr_err(stderr, "%s: cannot re-open %s: errno=%d (%s)\n",
+			name, filepath, errno, strerror(errno));
 		return;
 	}
 
-	inotify_exercise(filepath, path, "inotify_file", inotify_close_write_helper, IN_CLOSE_WRITE, (void*)&fd);
-	(void)rm_file(filepath);
+	inotify_exercise(name, filepath, path, "inotify_file", inotify_close_write_helper, IN_CLOSE_WRITE, (void*)&fd);
+	(void)rm_file(name, filepath);
 	(void)close(fd);
 }
 
-static int inotify_close_nowrite_helper(const char *path, const void *fdptr)
+static int inotify_close_nowrite_helper(const char *name, const char *path, const void *fdptr)
 {
+	(void)name;
 	(void)path;
+
 	(void)close(*(int *)fdptr);
 	return 0;
 }
 
-static void inotify_close_nowrite_file(const char *path)
+static void inotify_close_nowrite_file(const char *name, const char *path)
 {
 	char filepath[PATH_MAX];
 	int fd;
 
 	mk_filename(filepath, PATH_MAX, path, "inotify_file");
-	if (mk_file(filepath, 4096) < 0)
+	if (mk_file(name, filepath, 4096) < 0)
 		return;
 
 	if ((fd = open(filepath, O_RDONLY)) < 0) {
-		pr_err(stderr, "cannot re-open %s: errno=%d (%s)\n",
-			filepath, errno, strerror(errno));
-		(void)rm_file(filepath);
+		pr_err(stderr, "%s: cannot re-open %s: errno=%d (%s)\n",
+			name, filepath, errno, strerror(errno));
+		(void)rm_file(name, filepath);
 		return;
 	}
 
-	inotify_exercise(filepath, path, "inotify_file", inotify_close_nowrite_helper, IN_CLOSE_NOWRITE, (void*)&fd);
-	(void)rm_file(filepath);
+	inotify_exercise(name, filepath, path, "inotify_file", inotify_close_nowrite_helper, IN_CLOSE_NOWRITE, (void*)&fd);
+	(void)rm_file(name, filepath);
 	(void)close(fd);
 }
 
@@ -634,7 +653,7 @@ int stress_inotify(
 		return EXIT_FAILURE;
 	do {
 		for (i = 0; opt_do_run && inotify_stressors[i].func; i++)
-			inotify_stressors[i].func(dirname);
+			inotify_stressors[i].func(name, dirname);
 		(*counter)++;
 	} while (opt_do_run && (!max_ops || *counter < max_ops));
 	(void)stress_temp_dir_rm(name, pid, instance);
