@@ -24,6 +24,8 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -35,26 +37,47 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-
-#include "stress-ng.h"
+#if defined(STRESS_MALLOPT)
+#include <malloc.h>
+#endif
 
 static size_t opt_malloc_bytes = DEFAULT_MALLOC_BYTES;
+static bool set_malloc_bytes = false;
+
 static size_t opt_malloc_max = DEFAULT_MALLOC_MAX;
+static bool set_malloc_max = false;
+
+#if defined(STRESS_MALLOPT)
+static int opt_malloc_threshold = DEFAULT_MALLOC_THRESHOLD;
+static bool set_malloc_threshold = false;
+#endif
 
 void stress_set_malloc_bytes(const char *optarg)
 {
+	set_malloc_bytes = true;
 	opt_malloc_bytes = (size_t)get_uint64_byte(optarg);
 	check_range("malloc-bytes", opt_malloc_bytes,
-	MIN_MALLOC_BYTES, MAX_MALLOC_BYTES);
+		MIN_MALLOC_BYTES, MAX_MALLOC_BYTES);
 }
 
 void stress_set_malloc_max(const char *optarg)
 {
+	set_malloc_max = true;
 	opt_malloc_max = (size_t)get_uint64_byte(optarg);
 	check_range("malloc-max", opt_malloc_max,
-	MIN_MALLOC_MAX, MAX_MALLOC_MAX);
+		MIN_MALLOC_MAX, MAX_MALLOC_MAX);
 }
 
+#if defined(STRESS_MALLOPT)
+void stress_set_malloc_threshold(const char *optarg)
+{
+	set_malloc_threshold = true;
+	opt_malloc_threshold = (size_t)get_uint64_byte(optarg);
+	check_range("malloc-threshold", opt_malloc_threshold,
+		MIN_MALLOC_THRESHOLD, MAX_MALLOC_THRESHOLD);
+}
+#endif
+
 /*
  *  stress_alloc_size()
  *	get a new allocation size, ensuring
@@ -81,6 +104,25 @@ int stress_malloc(
 	pid_t pid;
 	uint32_t restarts = 0, nomems = 0;
 
+	if (!set_malloc_bytes) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_malloc_bytes = MAX_MALLOC_BYTES;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_malloc_bytes = MIN_MALLOC_BYTES;
+	}
+
+	if (!set_malloc_max) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_malloc_max = MAX_MALLOC_MAX;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_malloc_max = MIN_MALLOC_MAX;
+	}
+
+#if defined(STRESS_MALLOPT)
+	if (set_malloc_threshold)
+		mallopt(M_MMAP_THRESHOLD, opt_malloc_threshold);
+#endif
+
 again:
 	pid = fork();
 	if (pid < 0) {
@@ -96,7 +138,7 @@ again:
 					name, errno, strerror(errno));
 			(void)kill(pid, SIGTERM);
 			(void)kill(pid, SIGKILL);
-			waitpid(pid, &status, 0);
+			(void)waitpid(pid, &status, 0);
 		} else if (WIFSIGNALED(status)) {
 			pr_dbg(stderr, "%s: child died: %d (instance %d)\n",
 				name, WTERMSIG(status), instance);
@@ -150,7 +192,6 @@ again:
 						(void)mincore_touch_pages(addr[i], len);
 						(*counter)++;
 					}
-					
 				}
 			} else {
 				/* 50% free, 50% alloc */
