@@ -39,7 +39,11 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/mman.h>
+#if defined (__linux__)
+#include <sys/syscall.h>
+#endif
 #include <fcntl.h>
+#include <errno.h>
 
 #define _GNU_SOURCE
 /* GNU HURD */
@@ -56,33 +60,61 @@
 #define SOCKET_BUF		(8192)		/* Socket I/O buffer size */
 #define UDP_BUF			(1024)		/* UDP I/O buffer size */
 
+/* debug output bitmasks */
+#define PR_ERROR		0x0000000001ULL /* Print errors */
+#define PR_INFO			0x0000000002ULL /* Print info */
+#define PR_DEBUG		0x0000000004ULL /* Print debug */
+#define PR_FAIL			0x0000000008ULL /* Print test failure message */
+#define PR_ALL			(PR_ERROR | PR_INFO | PR_DEBUG | PR_FAIL)
+
 /* Option bit masks */
-#define OPT_FLAGS_AFFINITY_RAND	0x00000001	/* Change affinity randomly */
-#define OPT_FLAGS_DRY_RUN	0x00000002	/* Don't actually run */
-#define OPT_FLAGS_METRICS	0x00000004	/* Dump metrics at end */
-#define OPT_FLAGS_VM_KEEP	0x00000008	/* Don't keep re-allocating */
-#define OPT_FLAGS_RANDOM	0x00000010	/* Randomize */
-#define OPT_FLAGS_SET		0x00000020	/* Set if user specifies stress procs */
-#define OPT_FLAGS_KEEP_NAME	0x00000040	/* Keep stress names to stress-ng */
-#define OPT_FLAGS_UTIME_FSYNC	0x00000080	/* fsync after utime modification */
-#define OPT_FLAGS_METRICS_BRIEF	0x00000100	/* dump brief metrics */
-#define OPT_FLAGS_VERIFY	0x00000200	/* verify mode */
-#define OPT_FLAGS_MMAP_MADVISE	0x00000400	/* enable random madvise settings */
-#define OPT_FLAGS_MMAP_MINCORE	0x00000800	/* mincore force pages into mem */
-#define OPT_FLAGS_TIMES		0x00001000	/* user/system time summary */
-#define OPT_FLAGS_CACHE_FLUSH	0x00004000	/* cache flush */
-#define OPT_FLAGS_CACHE_FENCE	0x00008000	/* cache fence */
-#define OPT_FLAGS_CACHE_MASK	(OPT_FLAGS_CACHE_FLUSH | OPT_FLAGS_CACHE_FENCE)
-#define OPT_FLAGS_MMAP_FILE	0x00010000	/* mmap onto a file */
-#define OPT_FLAGS_MMAP_ASYNC	0x00020000	/* mmap onto a file */
-#define OPT_FLAGS_MMAP_MPROTECT	0x00040000	/* mmap mprotect enabled */
-#define OPT_FLAGS_LOCKF_NONBLK	0x00080000	/* Non-blocking lockf */
-#define OPT_FLAGS_MINCORE_RAND	0x00100000	/* mincore randomize */
-#define OPT_FLAGS_BRK_NOTOUCH	0x00200000	/* brk, don't touch page */
-#define OPT_FLAGS_HDD_SYNC	0x00400000	/* HDD O_SYNC */
-#define OPT_FLAGS_HDD_DSYNC	0x00800000	/* HDD O_DYNC */
-#define OPT_FLAGS_HDD_DIRECT	0x01000000	/* HDD O_DIRECT */
-#define OPT_FLAGS_HDD_NOATIME	0x02000000	/* HDD O_NOATIME */
+#define OPT_FLAGS_AFFINITY_RAND	0x0000000010ULL	/* Change affinity randomly */
+#define OPT_FLAGS_DRY_RUN	0x0000000020ULL	/* Don't actually run */
+#define OPT_FLAGS_METRICS	0x0000000040ULL	/* Dump metrics at end */
+#define OPT_FLAGS_VM_KEEP	0x0000000080ULL	/* Don't keep re-allocating */
+#define OPT_FLAGS_RANDOM	0x0000000100ULL	/* Randomize */
+#define OPT_FLAGS_SET		0x0000000200ULL	/* Set if user specifies stress procs */
+#define OPT_FLAGS_KEEP_NAME	0x0000000400ULL	/* Keep stress names to stress-ng */
+#define OPT_FLAGS_UTIME_FSYNC	0x0000000800ULL	/* fsync after utime modification */
+#define OPT_FLAGS_METRICS_BRIEF	0x0000001000ULL	/* dump brief metrics */
+#define OPT_FLAGS_VERIFY	0x0000002000ULL	/* verify mode */
+#define OPT_FLAGS_MMAP_MADVISE	0x0000004000ULL	/* enable random madvise settings */
+#define OPT_FLAGS_MMAP_MINCORE	0x0000008000ULL	/* mincore force pages into mem */
+#define OPT_FLAGS_TIMES		0x0000010000ULL	/* user/system time summary */
+#define OPT_FLAGS_CACHE_PREFETCH 0x0000020000ULL /* cache prefetch */
+#define OPT_FLAGS_CACHE_FLUSH	0x0000040000ULL	/* cache flush */
+#define OPT_FLAGS_CACHE_FENCE	0x0000080000ULL	/* cache fence */
+#define OPT_FLAGS_CACHE_MASK	(OPT_FLAGS_CACHE_FLUSH | \
+				 OPT_FLAGS_CACHE_FENCE | \
+				 OPT_FLAGS_CACHE_PREFETCH)
+#define OPT_FLAGS_MMAP_FILE	0x0000100000ULL	/* mmap onto a file */
+#define OPT_FLAGS_MMAP_ASYNC	0x0000200000ULL	/* mmap file asynchronous I/O */
+#define OPT_FLAGS_MMAP_MPROTECT	0x0000400000ULL	/* mmap mprotect enabled */
+#define OPT_FLAGS_LOCKF_NONBLK	0x0000800000ULL	/* Non-blocking lockf */
+#define OPT_FLAGS_MINCORE_RAND	0x0001000000ULL	/* mincore randomize */
+#define OPT_FLAGS_BRK_NOTOUCH	0x0002000000ULL	/* brk, don't touch page */
+#define OPT_FLAGS_HDD_SYNC	0x0004000000ULL	/* HDD O_SYNC */
+#define OPT_FLAGS_HDD_DSYNC	0x0008000000ULL	/* HDD O_DYNC */
+#define OPT_FLAGS_HDD_DIRECT	0x0010000000ULL	/* HDD O_DIRECT */
+#define OPT_FLAGS_HDD_NOATIME	0x0020000000ULL	/* HDD O_NOATIME */
+#define OPT_FLAGS_STACK_FILL	0x0040000000ULL	/* Fill stack */
+#define OPT_FLAGS_MINIMIZE	0x0080000000ULL	/* Minimize */
+#define OPT_FLAGS_MAXIMIZE	0x0100000000ULL	/* Maximize */
+#define OPT_FLAGS_MINMAX_MASK	(OPT_FLAGS_MINIMIZE | OPT_FLAGS_MAXIMIZE)
+#define OPT_FLAGS_SYSLOG	0x0200000000ULL	/* log test progress to syslog */
+#define OPT_FLAGS_AGGRESSIVE	0x0400000000ULL	/* aggressive mode enabled */
+
+#define OPT_FLAGS_AGGRESSIVE_MASK \
+	(OPT_FLAGS_AFFINITY_RAND | OPT_FLAGS_UTIME_FSYNC | \
+	 OPT_FLAGS_MMAP_MADVISE | OPT_FLAGS_MMAP_MINCORE | \
+	 OPT_FLAGS_CACHE_FLUSH | OPT_FLAGS_CACHE_FENCE |   \
+	 OPT_FLAGS_MMAP_FILE | OPT_FLAGS_MMAP_ASYNC |      \
+	 OPT_FLAGS_MMAP_MPROTECT | OPT_FLAGS_LOCKF_NONBLK |\
+	 OPT_FLAGS_MINCORE_RAND | OPT_FLAGS_HDD_SYNC |     \
+	 OPT_FLAGS_HDD_DSYNC | OPT_FLAGS_HDD_DIRECT |      \
+	 OPT_FLAGS_STACK_FILL | OPT_FLAGS_CACHE_PREFETCH | \
+	 OPT_FLAGS_AGGRESSIVE)
+
 
 /* Stressor classes */
 #define CLASS_CPU		0x00000001	/* CPU only */
@@ -95,20 +127,13 @@
 #define CLASS_INTERRUPT		0x00000080	/* interrupt floods */
 #define CLASS_OS		0x00000100	/* generic OS tests */
 
-/* debug output bitmasks */
-#define PR_ERROR		0x10000000	/* Print errors */
-#define PR_INFO			0x20000000	/* Print info */
-#define PR_DEBUG		0x40000000	/* Print debug */
-#define PR_FAIL			0x80000000	/* Print test failure message */
-#define PR_ALL			(PR_ERROR | PR_INFO | PR_DEBUG | PR_FAIL)
-
 /* Large prime to stride around large VM regions */
 #define PRIME_64		(0x8f0000000017116dULL)
 
 /* Logging helpers */
-extern int print(FILE *fp, const int flag,
+extern int print(FILE *fp, const uint64_t flag,
 	const char *const fmt, ...) __attribute__((format(printf, 3, 4)));
-extern void pr_failed(const int flag, const char *name, const char *what);
+extern void pr_failed(const uint64_t flag, const char *name, const char *what, const int err);
 
 #define pr_dbg(fp, fmt, args...)	print(fp, PR_DEBUG, fmt, ## args)
 #define pr_inf(fp, fmt, args...)	print(fp, PR_INFO, fmt, ## args)
@@ -116,8 +141,9 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 #define pr_fail(fp, fmt, args...)	print(fp, PR_FAIL, fmt, ## args)
 #define pr_tidy(fp, fmt, args...)	print(fp, opt_sigint ? PR_INFO : PR_DEBUG, fmt, ## args)
 
-#define pr_failed_err(name, what)	pr_failed(PR_ERROR, name, what)
-#define pr_failed_dbg(name, what)	pr_failed(PR_DEBUG, name, what)
+#define pr_failed_err(name, what)	pr_failed(PR_FAIL | PR_ERROR, name, what, errno)
+#define pr_failed_errno(name, what, e)	pr_failed(PR_FAIL | PR_ERROR, name, what, e)
+#define pr_failed_dbg(name, what)	pr_failed(PR_DEBUG, name, what, errno)
 
 #define ABORT_FAILURES		(5)
 
@@ -145,7 +171,7 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 #define DEFAULT_BSEARCH_SIZE	(64 * KB)
 
 #define MIN_DENTRIES		(1)
-#define MAX_DENTRIES		(100000000)
+#define MAX_DENTRIES		(1000000)
 #define DEFAULT_DENTRIES	(2048)
 
 #define MIN_EPOLL_PORT		(1024)
@@ -153,13 +179,17 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 #define DEFAULT_EPOLL_PORT	(6000)
 
 #define MIN_HDD_BYTES		(1 * MB)
-#define MAX_HDD_BYTES		(256 * GB)
+#define MAX_HDD_BYTES		(256ULL * GB)
 #define DEFAULT_HDD_BYTES	(1 * GB)
 
 #define MIN_HDD_WRITE_SIZE	(1)
 #define MAX_HDD_WRITE_SIZE	(4 * MB)
 #define DEFAULT_HDD_WRITE_SIZE	(64 * 1024)
 
+#define MIN_FALLOCATE_BYTES	(1 * MB)
+#define MAX_FALLOCATE_BYTES	(2 * GB)
+#define DEFAULT_FALLOCATE_BYTES	(1 * GB)
+
 #define MIN_FIFO_READERS	(1)
 #define MAX_FIFO_READERS	(64)
 #define DEFAULT_FIFO_READERS	(4)
@@ -185,11 +215,11 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 #define DEFAULT_HSEARCH_SIZE	(8 * KB)
 
 #define MIN_LEASE_BREAKERS	(1)
-#define MAX_LEASE_BREAKERS	(16)
+#define MAX_LEASE_BREAKERS	(64)
 #define DEFAULT_LEASE_BREAKERS	(1)
 
 #define MIN_LSEARCH_SIZE	(1 * KB)
-#define MAX_LSEARCH_SIZE	(4 * MB)
+#define MAX_LSEARCH_SIZE	(1 * MB)
 #define DEFAULT_LSEARCH_SIZE	(8 * KB)
 
 #define MIN_MALLOC_BYTES	(1 * KB)
@@ -200,6 +230,14 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 #define MAX_MALLOC_MAX		(256 * 1024)
 #define DEFAULT_MALLOC_MAX	(64 * KB)
 
+#define MIN_MALLOC_THRESHOLD	(1)
+#define MAX_MALLOC_THRESHOLD	(256 * MB)
+#define DEFAULT_MALLOC_THRESHOLD (128 * KB)
+
+#define MIN_MATRIX_SIZE		(16)
+#define MAX_MATRIX_SIZE		(4096)
+#define DEFAULT_MATRIX_SIZE	(256)
+
 #define MIN_MMAP_BYTES		(4 * KB)
 #define MAX_MMAP_BYTES		(1 * GB)
 #define DEFAULT_MMAP_BYTES	(256 * MB)
@@ -210,10 +248,10 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 
 #define MIN_PTHREAD		(1)
 #define MAX_PTHREAD		(30000)
-#define DEFAULT_PTHREAD		(16)
+#define DEFAULT_PTHREAD		(1024)
 
 #define MIN_QSORT_SIZE		(1 * KB)
-#define MAX_QSORT_SIZE		(64 * MB)
+#define MAX_QSORT_SIZE		(4 * MB)
 #define DEFAULT_QSORT_SIZE	(256 * KB)
 
 #define MIN_SENDFILE_SIZE	(1 * KB)
@@ -221,7 +259,7 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 #define DEFAULT_SENDFILE_SIZE	(4 * MB)
 
 #define MIN_SEEK_SIZE		(1 * MB)
-#define MAX_SEEK_SIZE 		(256 * GB)
+#define MAX_SEEK_SIZE 		(256ULL * GB)
 #define DEFAULT_SEEK_SIZE	(16 * MB)
 
 #define MIN_SEQUENTIAL		(0)
@@ -303,7 +341,7 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 #define STRESS_X86	1
 #endif
 
-#if defined(__GNUC__) && defined (__GNUC_MINOR__) && \
+#if defined(__GNUC__) && defined(__GNUC_MINOR__) && \
     (__GNUC__ >= 4) && (__GNUC_MINOR__ >= 6)
 #define STRESS_VECTOR	1
 #endif
@@ -317,6 +355,23 @@ extern void pr_failed(const int flag, const char *name, const char *what);
 #define STRESS_INT128	1
 #endif
 
+#if defined(__linux__)
+#define STRESS_IONICE
+#endif
+
+#if (_BSD_SOURCE || _SVID_SOURCE) && !defined(__gnu_hurd__)
+#define STRESS_PAGE_IN
+#endif
+
+#if defined(__linux__)
+#define STRESS_IOPRIO
+#endif
+
+#if defined(__GNUC__) && defined(__linux__)
+#define STRESS_MALLOPT
+#endif
+
+
 /* stress process prototype */
 typedef int (*stress_func)(uint64_t *const counter, const uint32_t instance,
 		    const uint64_t max_ops, const char *name);
@@ -370,11 +425,14 @@ typedef struct {
 
 /* Stress tests */
 typedef enum {
+	STRESS_START = -1,
 #if defined(__linux__)
-	STRESS_AFFINITY = 0,
+	__STRESS_AFFINITY,
+#define STRESS_AFFINITY __STRESS_AFFINITY
 #endif
 #if defined(__linux__)
-	STRESS_AIO,
+	__STRESS_AIO,
+#define STRESS_AIO __STRESS_AIO
 #endif
 	STRESS_BRK,
 	STRESS_BSEARCH,
@@ -382,20 +440,24 @@ typedef enum {
 	STRESS_CACHE,
 	STRESS_CHMOD,
 #if _POSIX_C_SOURCE >= 199309L
-	STRESS_CLOCK,
+	__STRESS_CLOCK,
+#define STRESS_CLOCK __STRESS_CLOCK
 #endif
 	STRESS_CPU,
 	STRESS_DENTRY,
 	STRESS_DIR,
 	STRESS_DUP,
 #if defined(__linux__)
-	STRESS_EPOLL,
+	__STRESS_EPOLL,
+#define STRESS_EPOLL __STRESS_EPOLL
 #endif
 #if defined(__linux__)
-	STRESS_EVENTFD,
+	__STRESS_EVENTFD,
+#define STRESS_EVENTFD __STRESS_EVENTFD
 #endif
 #if _XOPEN_SOURCE >= 600 || _POSIX_C_SOURCE >= 200112L
-	STRESS_FALLOCATE,
+	__STRESS_FALLOCATE,
+#define STRESS_FALLOCATE __STRESS_FALLOCATE
 #endif
 	STRESS_FAULT,
 	STRESS_FIFO,
@@ -403,39 +465,53 @@ typedef enum {
 	STRESS_FORK,
 	STRESS_FSTAT,
 #if defined(__linux__)
-	STRESS_FUTEX,
+	__STRESS_FUTEX,
+#define STRESS_FUTEX __STRESS_FUTEX
 #endif
 	STRESS_GET,
 	STRESS_HDD,
 	STRESS_HSEARCH,
 #if defined(__linux__)
-	STRESS_INOTIFY,
+	__STRESS_INOTIFY,
+#define STRESS_INOTIFY __STRESS_INOTIFY
 #endif
 	STRESS_IOSYNC,
+#if defined(__linux__) && defined(__NR_kcmp)
+	__STRESS_KCMP,
+#define STRESS_KCMP __STRESS_KCMP
+#endif
 	STRESS_KILL,
 #if defined(F_SETLEASE) && defined(F_WRLCK) && defined(F_UNLCK)
-	STRESS_LEASE,
+	__STRESS_LEASE,
+#define STRESS_LEASE __STRESS_LEASE
 #endif
 	STRESS_LINK,
 #if _BSD_SOURCE || _SVID_SOURCE || _XOPEN_SOURCE >= 500 || \
      (_XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED)
-	STRESS_LOCKF,
+	__STRESS_LOCKF,
+#define STRESS_LOCKF __STRESS_LOCKF
 #endif
+	STRESS_LONGJMP,
 	STRESS_LSEARCH,
 	STRESS_MALLOC,
+	STRESS_MATRIX,
 	STRESS_MEMCPY,
 #if (_BSD_SOURCE || _SVID_SOURCE) && !defined(__gnu_hurd__)
-	STRESS_MINCORE,
+	__STRESS_MINCORE,
+#define STRESS_MINCORE __STRESS_MINCORE
 #endif
 	STRESS_MMAP,
 #if defined(__linux__)
-	STRESS_MREMAP,
+	__STRESS_MREMAP,
+#define STRESS_MREMAP __STRESS_MREMAP
 #endif
 #if !defined(__gnu_hurd__)
-	STRESS_MSG,
+	__STRESS_MSG,
+#define STRESS_MSG __STRESS_MSG
 #endif
 #if defined(__linux__)
-	STRESS_MQ,
+	__STRESS_MQ,
+#define STRESS_MQ __STRESS_MQ
 #endif
 	STRESS_NICE,
 	STRESS_NULL,
@@ -443,68 +519,84 @@ typedef enum {
 	STRESS_PIPE,
 	STRESS_POLL,
 #if defined(__linux__)
-	STRESS_PROCFS,
+	__STRESS_PROCFS,
+#define STRESS_PROCFS __STRESS_PROCFS
 #endif
 	STRESS_PTHREAD,
 	STRESS_QSORT,
-#if defined(STRESS_X86)
-	STRESS_RDRAND,
+#if defined(STRESS_X86) && !defined(__OpenBSD__)
+	__STRESS_RDRAND,
+#define STRESS_RDRAND __STRESS_RDRAND
 #endif
 	STRESS_RENAME,
 	STRESS_SEEK,
 	STRESS_SEMAPHORE_POSIX,
 #if !defined(__gnu_hurd__)
-	STRESS_SEMAPHORE_SYSV,
+	__STRESS_SEMAPHORE_SYSV,
+#define STRESS_SEMAPHORE_SYSV __STRESS_SEMAPHORE_SYSV
 #endif
 #if defined(__linux__)
-	STRESS_SENDFILE,
+	__STRESS_SENDFILE,
+#define STRESS_SENDFILE __STRESS_SENDFILE
 #endif
 	STRESS_SHM_SYSV,
 #if defined(__linux__)
-	STRESS_SIGFD,
+	__STRESS_SIGFD,
+#define STRESS_SIGFD __STRESS_SIGFD
 #endif
 	STRESS_SIGFPE,
 #if _POSIX_C_SOURCE >= 199309L && !defined(__gnu_hurd__)
-	STRESS_SIGQUEUE,
+	__STRESS_SIGQUEUE,
+#define STRESS_SIGQUEUE __STRESS_SIGQUEUE
 #endif
 	STRESS_SIGSEGV,
 	STRESS_SOCKET,
 #if defined(__linux__)
-	STRESS_SPLICE,
+	__STRESS_SPLICE,
+#define STRESS_SPLICE __STRESS_SPLICE
 #endif
 	STRESS_STACK,
 	STRESS_SWITCH,
 	STRESS_SYMLINK,
 	STRESS_SYSINFO,
 #if defined(__linux__)
-	STRESS_TIMER,
+	__STRESS_TIMER,
+#define STRESS_TIMER __STRESS_TIMER
 #endif
 	STRESS_TSEARCH,
 	STRESS_UDP,
 #if defined(__linux__) || defined(__gnu_hurd__)
-	STRESS_URANDOM,
+	__STRESS_URANDOM,
+#define STRESS_URANDOM __STRESS_URANDOM
 #endif
 	STRESS_UTIME,
 #if defined(STRESS_VECTOR)
-	STRESS_VECMATH,
+	__STRESS_VECMATH,
+#define STRESS_VECMATH __STRESS_VECMATH
 #endif
 #if  _BSD_SOURCE || \
     (_XOPEN_SOURCE >= 500 || _XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED) && \
     !(_POSIX_C_SOURCE >= 200809L || _XOPEN_SOURCE >= 700)
-	STRESS_VFORK,
+	__STRESS_VFORK,
+#define STRESS_VFORK __STRESS_VFORK
 #endif
 	STRESS_VM,
-#if defined(__linux__)
-	STRESS_VM_RW,
+#if defined(__linux__) && \
+    defined(__NR_process_vm_readv) && defined(__NR_process_vm_writev)
+	__STRESS_VM_RW,
+#define STRESS_VM_RW __STRESS_VM_RW
 #endif
 #if defined(__linux__)
-	STRESS_VM_SPLICE,
+	__STRESS_VM_SPLICE,
+#define STRESS_VM_SPLICE __STRESS_VM_SPLICE
 #endif
 #if !defined(__gnu_hurd__) && !defined(__NetBSD__)
-	STRESS_WAIT,
+	__STRESS_WAIT,
+#define STRESS_WAIT __STRESS_WAIT
 #endif
-#if defined (_POSIX_PRIORITY_SCHEDULING)
-	STRESS_YIELD,
+#if defined(_POSIX_PRIORITY_SCHEDULING)
+	__STRESS_YIELD,
+#define STRESS_YIELD __STRESS_YIELD
 #endif
 	STRESS_ZERO,
 	/* STRESS_MAX must be last one */
@@ -523,7 +615,7 @@ typedef enum {
 	OPT_HDD = 'd',
 	OPT_DENTRY = 'D',
 	OPT_FORK = 'f',
-#if _XOPEN_SOURCE >= 600 || _POSIX_C_SOURCE >= 200112L
+#if defined(STRESS_FALLOCATE)
 	OPT_FALLOCATE = 'F',
 #endif
 	OPT_IOSYNC = 'i',
@@ -542,10 +634,10 @@ typedef enum {
 	OPT_SWITCH = 's',
 	OPT_SOCKET = 'S',
 	OPT_TIMEOUT = 't',
-#if defined (__linux__)
+#if defined(STRESS_TIMER)
 	OPT_TIMER = 'T',
 #endif
-#if defined (__linux__) || defined(__gnu_hurd__)
+#if defined(STRESS_URANDOM)
 	OPT_URANDOM = 'u',
 #endif
 	OPT_VERBOSE = 'v',
@@ -554,17 +646,22 @@ typedef enum {
 
 	/* Long options only */
 
-#if defined(__linux__)
-	OPT_AFFINITY = 0x80,
+	OPT_LONG_OPS_START = 0x7f,
+
+#if defined(STRESS_AFFINITY)
+	OPT_AFFINITY,
 	OPT_AFFINITY_OPS,
 	OPT_AFFINITY_RAND,
 #endif
 
-#if defined (__linux__)
+	OPT_AGGRESSIVE,
+
+#if defined(STRESS_AIO)
 	OPT_AIO,
 	OPT_AIO_OPS,
 	OPT_AIO_REQUESTS,
 #endif
+
 	OPT_BRK,
 	OPT_BRK_OPS,
 	OPT_BRK_NOTOUCH,
@@ -578,13 +675,14 @@ typedef enum {
 
 	OPT_CLASS,
 	OPT_CACHE_OPS,
+	OPT_CACHE_PREFETCH,
 	OPT_CACHE_FLUSH,
 	OPT_CACHE_FENCE,
 
 	OPT_CHMOD,
 	OPT_CHMOD_OPS,
 
-#if _POSIX_C_SOURCE >= 199309L
+#if defined(STRESS_CLOCK)
 	OPT_CLOCK,
 	OPT_CLOCK_OPS,
 #endif
@@ -602,7 +700,7 @@ typedef enum {
 	OPT_DUP,
 	OPT_DUP_OPS,
 
-#if defined(__linux__)
+#if defined(STRESS_EPOLL)
 	OPT_EPOLL,
 	OPT_EPOLL_OPS,
 	OPT_EPOLL_PORT,
@@ -614,13 +712,14 @@ typedef enum {
 	OPT_HDD_OPS,
 	OPT_HDD_OPTS,
 
-#if defined(__linux__)
+#if defined(STRESS_EVENTFD)
 	OPT_EVENTFD,
 	OPT_EVENTFD_OPS,
 #endif
 
-#if _XOPEN_SOURCE >= 600 || _POSIX_C_SOURCE >= 200112L
+#if defined(STRESS_FALLOCATE)
 	OPT_FALLOCATE_OPS,
+	OPT_FALLOCATE_BYTES,
 #endif
 	OPT_FAULT,
 	OPT_FAULT_OPS,
@@ -649,22 +748,27 @@ typedef enum {
 	OPT_HSEARCH_OPS,
 	OPT_HSEARCH_SIZE,
 
-#if defined (__linux__)
+#if defined(STRESS_INOTIFY)
 	OPT_INOTIFY,
 	OPT_INOTIFY_OPS,
 #endif
 
-#if defined (__linux__)
+#if defined(STRESS_IONICE)
 	OPT_IONICE_CLASS,
 	OPT_IONICE_LEVEL,
 #endif
 
 	OPT_IOSYNC_OPS,
 
+#if defined(STRESS_KCMP)
+	OPT_KCMP,
+	OPT_KCMP_OPS,
+#endif
+
 	OPT_KILL,
 	OPT_KILL_OPS,
 
-#if defined(F_SETLEASE) && defined(F_WRLCK) && defined(F_UNLCK)
+#if defined(STRESS_LEASE)
 	OPT_LEASE,
 	OPT_LEASE_OPS,
 	OPT_LEASE_BREAKERS,
@@ -673,13 +777,15 @@ typedef enum {
 	OPT_LINK,
 	OPT_LINK_OPS,
 
-#if _BSD_SOURCE || _SVID_SOURCE || _XOPEN_SOURCE >= 500 || \
-     (_XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED)
+#if defined(STRESS_LOCKF)
 	OPT_LOCKF,
 	OPT_LOCKF_OPS,
 	OPT_LOCKF_NONBLOCK,
 #endif
 
+	OPT_LONGJMP,
+	OPT_LONGJMP_OPS,
+
 	OPT_LSEARCH,
 	OPT_LSEARCH_OPS,
 	OPT_LSEARCH_SIZE,
@@ -688,18 +794,30 @@ typedef enum {
 	OPT_MALLOC_OPS,
 	OPT_MALLOC_BYTES,
 	OPT_MALLOC_MAX,
+#if defined(STRESS_MALLOPT)
+	OPT_MALLOC_THRESHOLD,
+#endif
+
+	OPT_MATRIX,
+	OPT_MATRIX_OPS,
+	OPT_MATRIX_SIZE,
+	OPT_MATRIX_METHOD,
+
+	OPT_MAXIMIZE,
 
 	OPT_MEMCPY,
 	OPT_MEMCPY_OPS,
 
 	OPT_METRICS_BRIEF,
 
-#if (_BSD_SOURCE || _SVID_SOURCE) && !defined(__gnu_hurd__)
+#if defined(STRESS_MINCORE)
 	OPT_MINCORE,
 	OPT_MINCORE_OPS,
 	OPT_MINCORE_RAND,
 #endif
 
+	OPT_MINIMIZE,
+
 	OPT_MMAP,
 	OPT_MMAP_OPS,
 	OPT_MMAP_BYTES,
@@ -707,7 +825,7 @@ typedef enum {
 	OPT_MMAP_ASYNC,
 	OPT_MMAP_MPROTECT,
 
-#if defined(__linux__)
+#if defined(STRESS_MREMAP)
 	OPT_MREMAP,
 	OPT_MREMAP_OPS,
 	OPT_MREMAP_BYTES,
@@ -716,7 +834,7 @@ typedef enum {
 	OPT_MSG,
 	OPT_MSG_OPS,
 
-#if defined(__linux__)
+#if defined(STRESS_MQ)
 	OPT_MQ,
 	OPT_MQ_OPS,
 	OPT_MQ_SIZE,
@@ -732,7 +850,7 @@ typedef enum {
 
 	OPT_OPEN_OPS,
 
-#if (_BSD_SOURCE || _SVID_SOURCE) && !defined(__gnu_hurd__)
+#if defined(STRESS_PAGE_IN)
 	OPT_PAGE_IN,
 #endif
 
@@ -740,8 +858,10 @@ typedef enum {
 
 	OPT_POLL_OPS,
 
+#if defined(STRESS_PROCFS)
 	OPT_PROCFS,
 	OPT_PROCFS_OPS,
+#endif
 
 	OPT_PTHREAD,
 	OPT_PTHREAD_OPS,
@@ -751,8 +871,10 @@ typedef enum {
 	OPT_QSORT_OPS,
 	OPT_QSORT_INTEGERS,
 
+#if defined(STRESS_RDRAND)
 	OPT_RDRAND,
 	OPT_RDRAND_OPS,
+#endif
 
 	OPT_RENAME_OPS,
 
@@ -763,15 +885,17 @@ typedef enum {
 	OPT_SEEK_OPS,
 	OPT_SEEK_SIZE,
 
+#if defined(STRESS_SENDFILE)
 	OPT_SENDFILE,
 	OPT_SENDFILE_OPS,
 	OPT_SENDFILE_SIZE,
+#endif
 
 	OPT_SEMAPHORE_POSIX,
 	OPT_SEMAPHORE_POSIX_OPS,
 	OPT_SEMAPHORE_POSIX_PROCS,
 
-#if !defined(__gnu_hurd__)
+#if defined(STRESS_SEMAPHORE_SYSV)
 	OPT_SEMAPHORE_SYSV,
 	OPT_SEMAPHORE_SYSV_OPS,
 	OPT_SEMAPHORE_SYSV_PROCS,
@@ -784,8 +908,10 @@ typedef enum {
 
 	OPT_SEQUENTIAL,
 
+#if defined(STRESS_SIGFD)
 	OPT_SIGFD,
 	OPT_SIGFD_OPS,
+#endif
 
 	OPT_SIGFPE,
 	OPT_SIGFPE_OPS,
@@ -793,7 +919,7 @@ typedef enum {
 	OPT_SIGSEGV,
 	OPT_SIGSEGV_OPS,
 
-#if _POSIX_C_SOURCE >= 199309L
+#if defined(STRESS_SIGQUEUE)
 	OPT_SIGQUEUE,
 	OPT_SIGQUEUE_OPS,
 #endif
@@ -804,7 +930,7 @@ typedef enum {
 
 	OPT_SWITCH_OPS,
 
-#if defined(__linux__)
+#if defined(STRESS_SPLICE)
 	OPT_SPLICE,
 	OPT_SPLICE_OPS,
 	OPT_SPLICE_BYTES,
@@ -812,6 +938,7 @@ typedef enum {
 
 	OPT_STACK,
 	OPT_STACK_OPS,
+	OPT_STACK_FILL,
 
 	OPT_SYMLINK,
 	OPT_SYMLINK_OPS,
@@ -819,7 +946,9 @@ typedef enum {
 	OPT_SYSINFO,
 	OPT_SYSINFO_OPS,
 
-#if defined (__linux__)
+	OPT_SYSLOG,
+
+#if defined(STRESS_TIMER)
 	OPT_TIMER_OPS,
 	OPT_TIMER_FREQ,
 #endif
@@ -835,23 +964,21 @@ typedef enum {
 	OPT_UDP_PORT,
 	OPT_UDP_DOMAIN,
 
-#if defined (__linux__) || defined(__gnu_hurd__)
+#if defined(STRESS_URANDOM)
 	OPT_URANDOM_OPS,
 #endif
 	OPT_UTIME,
 	OPT_UTIME_OPS,
 	OPT_UTIME_FSYNC,
 
-#if defined(STRESS_VECTOR)
+#if defined(STRESS_VECMATH)
 	OPT_VECMATH,
 	OPT_VECMATH_OPS,
 #endif
 
 	OPT_VERIFY,
 
-#if  _BSD_SOURCE || \
-     (_XOPEN_SOURCE >= 500 || _XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED) && \
-     !(_POSIX_C_SOURCE >= 200809L || _XOPEN_SOURCE >= 700)
+#if defined(STRESS_VFORK)
 	OPT_VFORK,
 	OPT_VFORK_OPS,
 	OPT_VFORK_MAX,
@@ -869,24 +996,24 @@ typedef enum {
 	OPT_VM_OPS,
 	OPT_VM_METHOD,
 
-#if defined(__linux__)
+#if defined(STRESS_VM_RW)
 	OPT_VM_RW,
 	OPT_VM_RW_OPS,
 	OPT_VM_RW_BYTES,
 #endif
 
-#if defined(__linux__)
+#if defined(STRESS_VM_SPLICE)
 	OPT_VM_SPLICE,
 	OPT_VM_SPLICE_OPS,
 	OPT_VM_SPLICE_BYTES,
 #endif
 
-#if !defined(__gnu_hurd__) && !defined(__NetBSD__)
+#if defined(STRESS_WAIT)
 	OPT_WAIT,
 	OPT_WAIT_OPS,
 #endif
 
-#if defined (_POSIX_PRIORITY_SCHEDULING)
+#if defined(STRESS_YIELD)
 	OPT_YIELD_OPS,
 #endif
 
@@ -920,8 +1047,8 @@ typedef struct {
 extern const char *app_name;		/* Name of application */
 extern shared_t *shared;		/* shared memory */
 extern uint64_t	opt_timeout;		/* timeout in seconds */
-extern int32_t	opt_flags;		/* option flags */
-extern uint64_t opt_sequential;		/* Number of sequential iterations */
+extern uint64_t	opt_flags;		/* option flags */
+extern int32_t opt_sequential;		/* Number of sequential iterations */
 extern volatile bool opt_do_run;	/* false to exit stressor */
 extern volatile bool opt_sigint;	/* true if stopped by SIGINT */
 extern mwc_t __mwc;			/* internal mwc random state */
@@ -957,13 +1084,13 @@ static inline void mfence(void)
 
 #else
 
-#define clflush(ptr)	/* No-op */
-#define mfence()	/* No-op */
+#define clflush(ptr)	do { } while (0) /* No-op */
+#define mfence()	do { } while (0) /* No-op */
 
 #endif
 
 /*
- *  mwc() 
+ *  mwc()
  *      Multiply-with-carry random numbers
  *      fast pseudo random number generator, see
  *      http://www.cse.yorku.ca/~oz/marsaglia-rng.html
@@ -983,10 +1110,9 @@ extern double timeval_to_double(const struct timeval *tv);
 extern double time_now(void);
 
 /* Misc settings helpers */
-extern void set_oom_adjustment(const char *name, bool killable);
+extern void set_oom_adjustment(const char *name, const bool killable);
 extern void set_sched(const int sched, const int sched_priority);
 extern void set_iopriority(const int class, const int level);
-extern void set_oom_adjustment(const char *name, bool killable);
 extern void set_coredump(const char *name);
 extern void set_proc_name(const char *name);
 
@@ -1012,8 +1138,8 @@ extern long stress_get_ticks_per_second(void);
 extern void set_max_limits(void);
 
 /* Memory tweaking */
-extern int madvise_random(void *addr, size_t length);
-extern int mincore_touch_pages(void *buf, size_t buf_len);
+extern int madvise_random(void *addr, const size_t length);
+extern int mincore_touch_pages(void *buf, const size_t buf_len);
 
 /* Mounts */
 extern int mount_add(char *mnts[], const int max, int *n, const char *name);
@@ -1045,6 +1171,7 @@ extern void stress_set_dentries(const char *optarg);
 extern int  stress_set_dentry_order(const char *optarg);
 extern void stress_set_epoll_port(const char *optarg);
 extern int  stress_set_epoll_domain(const char *optarg);
+extern void stress_set_fallocate_bytes(const char *optarg);
 extern void stress_set_fifo_readers(const char *optarg);
 extern void stress_set_fork_max(const char *optarg);
 extern void stress_set_fstat_dir(const char *optarg);
@@ -1056,11 +1183,15 @@ extern void stress_set_lease_breakers(const char *optarg);
 extern void stress_set_lsearch_size(const char *optarg);
 extern void stress_set_malloc_bytes(const char *optarg);
 extern void stress_set_malloc_max(const char *optarg);
+extern void stress_set_malloc_threshold(const char *optarg);
+extern int stress_set_matrix_method(const char *name);
+extern void stress_set_matrix_size(const char *optarg);
 extern void stress_set_mmap_bytes(const char *optarg);
 extern void stress_set_mremap_bytes(const char *optarg);
 extern void stress_set_mq_size(const char *optarg);
 extern void stress_set_pthread_max(const char *optarg);
 extern void stress_set_qsort_size(const void *optarg);
+extern int  stress_rdrand_supported(void);
 extern void stress_set_seek_size(const char *optarg);
 extern void stress_set_sendfile_size(const char *optarg);
 extern void stress_set_semaphore_posix_procs(const char *optarg);
@@ -1113,12 +1244,15 @@ STRESS(stress_futex);
 STRESS(stress_get);
 STRESS(stress_inotify);
 STRESS(stress_iosync);
+STRESS(stress_kcmp);
 STRESS(stress_kill);
 STRESS(stress_lease);
 STRESS(stress_link);
 STRESS(stress_lockf);
+STRESS(stress_longjmp);
 STRESS(stress_lsearch);
 STRESS(stress_malloc);
+STRESS(stress_matrix);
 STRESS(stress_memcpy);
 STRESS(stress_mincore);
 STRESS(stress_mmap);
