@@ -97,14 +97,19 @@ int stress_pipe(
 
 			n = read(pipefds[0], buf, sizeof(buf));
 			if (n <= 0) {
-				pr_failed_dbg(name, "read");
-				break;
+				if ((errno == EAGAIN) || (errno == EINTR))
+					continue;
+				if (errno) {
+					pr_failed_dbg(name, "read");
+					break;
+				}
+				continue;
 			}
 			if (!strcmp(buf, PIPE_STOP))
 				break;
 			if ((opt_flags & OPT_FLAGS_VERIFY) &&
 			    pipe_memchk(buf, val++, (size_t)n)) {
-				pr_fail(stderr, "pipe read error detected, failed to read expected data\n");
+				pr_fail(stderr, "%s: pipe read error detected, failed to read expected data\n", name);
 			}
 		}
 		(void)close(pipefds[0]);
@@ -117,10 +122,18 @@ int stress_pipe(
 		(void)close(pipefds[0]);
 
 		do {
+			ssize_t ret;
+
 			pipe_memset(buf, val++, sizeof(buf));
-			if (write(pipefds[1], buf, sizeof(buf)) < 0) {
-				pr_failed_dbg(name, "write");
-				break;
+			ret = write(pipefds[1], buf, sizeof(buf));
+			if (ret <= 0) {
+				if ((errno == EAGAIN) || (errno == EINTR))
+					continue;
+				if (errno) {
+					pr_failed_dbg(name, "write");
+					break;
+				}
+				continue;
 			}
 			(*counter)++;
 		} while (opt_do_run && (!max_ops || *counter < max_ops));
@@ -129,7 +142,7 @@ int stress_pipe(
 		if (write(pipefds[1], buf, sizeof(buf)) <= 0)
 			pr_failed_dbg(name, "termination write");
 		(void)kill(pid, SIGKILL);
-		waitpid(pid, &status, 0);
+		(void)waitpid(pid, &status, 0);
 	}
 	return EXIT_SUCCESS;
 }
