@@ -24,6 +24,10 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
+#if defined(STRESS_PROCFS)
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -33,11 +37,8 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include "stress-ng.h"
-
 #define PROC_BUF_SZ	(4096)
 
-#if defined(__linux__)
 /*
  *  stress_proc_read()
  *	read a proc file
@@ -52,9 +53,17 @@ static inline void stress_proc_read(const char *path)
 
 	/* Limit to 4K * 4K of data to read per file */
 	for (i = 0; i < 4096; i++) {
+		ssize_t ret;
+redo:
 		if (!opt_do_run)
 			break;
-		if (read(fd, buffer, PROC_BUF_SZ) < PROC_BUF_SZ)
+		ret = read(fd, buffer, PROC_BUF_SZ);
+		if (ret < 0) {
+			if ((errno == EAGAIN) || (errno == EINTR))
+				goto redo;
+			break;
+		}
+		if (ret < PROC_BUF_SZ)
 			break;
 	}
 	(void)close(fd);
