@@ -24,7 +24,9 @@
  */
 #define _GNU_SOURCE
 
-#if !defined(__gnu_hurd__)
+#include "stress-ng.h"
+
+#if defined(STRESS_SEMAPHORE_SYSV)
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,12 +39,19 @@
 #include <sys/ipc.h>
 #include <sys/sem.h>
 
-#include "stress-ng.h"
-
 static uint64_t opt_semaphore_sysv_procs = DEFAULT_SEMAPHORE_PROCS;
+static bool set_semaphore_sysv_procs = false;
+
+typedef union _semun {
+	int              val;	/* Value for SETVAL */
+	struct semid_ds *buf;	/* Buffer for IPC_STAT, IPC_SET */
+	unsigned short  *array;	/* Array for GETALL, SETALL */
+	struct seminfo  *__buf;	/* Buffer for IPC_INFO (Linux-specific) */
+} semun_t;
 
 void stress_set_semaphore_sysv_procs(const char *optarg)
 {
+	set_semaphore_sysv_procs = true;
 	opt_semaphore_sysv_procs = get_uint64_byte(optarg);
 	check_range("sem-procs", opt_semaphore_sysv_procs,
 		MIN_SEMAPHORE_PROCS, MAX_SEMAPHORE_PROCS);
@@ -66,8 +75,10 @@ void stress_semaphore_sysv_init(void)
 	}
 
 	if (shared->sem_sysv_id >= 0) {
-		unsigned short semval = 1;
-		if (semctl(shared->sem_sysv_id, 0, SETVAL, semval) == 0) {
+		semun_t arg;
+
+		arg.val = 1;
+		if (semctl(shared->sem_sysv_id, 0, SETVAL, arg) == 0) {
 			shared->sem_sysv_init = true;
 			return;
 		}
@@ -173,6 +184,13 @@ int stress_sem_sysv(
 
 	(void)instance;
 
+	if (!set_semaphore_sysv_procs) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_semaphore_sysv_procs = MAX_SEMAPHORE_PROCS;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_semaphore_sysv_procs = MIN_SEMAPHORE_PROCS;
+	}
+
 	if (!shared->sem_sysv_init) {
 		pr_err(stderr, "%s: aborting, semaphore not initialised\n", name);
 		return EXIT_FAILURE;
@@ -191,7 +209,7 @@ reap:
 			int status;
 
 			(void)kill(pids[i], SIGKILL);
-                        waitpid(pids[i], &status, 0);
+                        (void)waitpid(pids[i], &status, 0);
 		}
 	}
 
