@@ -24,15 +24,15 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
+#if defined(STRESS_VECMATH)
+
 #include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <unistd.h>
 
-#include "stress-ng.h"
-
-#if defined(STRESS_VECTOR)
-
 typedef int8_t  vint8_t  __attribute__ ((vector_size (16)));
 typedef int16_t vint16_t __attribute__ ((vector_size (16)));
 typedef int32_t vint32_t __attribute__ ((vector_size (16)));
@@ -47,6 +47,9 @@ typedef int64_t vint64_t __attribute__ ((vector_size (16)));
 	a = ~a;		\
 	a *= s;		\
 	a ^= c;		\
+	a <<= 1;	\
+	b >>= 1;	\
+	b += c;		\
 
 /*
  *  stress_vecmath()
@@ -100,7 +103,7 @@ int stress_vecmath(
 		0x0102030201020302ULL, 0x0302010203020102ULL };
 	vint64_t s64 = {
 		0x0000000000000001ULL, 0x0000000000000002ULL };
-		
+
 	do {
 		int i;
 		for (i = 1000; i; i--) {
