@@ -24,6 +24,10 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
+#if defined(STRESS_VM_RW)
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -32,21 +36,20 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/mman.h>
+#include <sys/uio.h>
 #include <errno.h>
 
-#if defined(__linux__)
-
-#include "stress-ng.h"
-
 typedef struct {
 	void *addr;	/* Buffer to read/write to */
 	uint8_t val;	/* Value to check */
 } addr_msg_t;
 
 static size_t opt_vm_rw_bytes = DEFAULT_VM_RW_BYTES;
+static bool set_vm_rw_bytes = false;
 
 void stress_set_vm_rw_bytes(const char *optarg)
 {
+	set_vm_rw_bytes = true;
 	opt_vm_rw_bytes = (size_t)get_uint64_byte(optarg);
 	check_range("vm-rw-bytes", opt_vm_rw_bytes,
 		MIN_VM_RW_BYTES, MAX_VM_RW_BYTES);
@@ -65,10 +68,18 @@ int stress_vm_rw(
 	pid_t pid;
 	int pipe_wr[2], pipe_rd[2];
 	const size_t page_size = stress_get_pagesize();
-	const size_t sz = opt_vm_rw_bytes & ~(page_size - 1);
+	size_t sz;
 
 	(void)instance;
 
+	if (!set_vm_rw_bytes) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_vm_rw_bytes = MAX_VM_RW_BYTES;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_vm_rw_bytes = MIN_VM_RW_BYTES;
+	}
+	sz = opt_vm_rw_bytes & ~(page_size - 1);
+
 	if (pipe(pipe_wr) < 0) {
 		pr_failed_dbg(name, "pipe");
 		return EXIT_FAILURE;
@@ -113,15 +124,24 @@ int stress_vm_rw(
 			msg_wr.val = 0;
 
 			/* Send address of buffer to parent */
+redo_wr1:
 			ret = write(pipe_wr[1], &msg_wr, sizeof(msg_wr));
 			if (ret < 0) {
+				if ((errno == EAGAIN) || (errno == EINTR))
+					goto redo_wr1;
 				if (errno != EBADF)
 					pr_failed_dbg(name, "write");
 				break;
 			}
-
+redo_rd1:
 			/* Wait for parent to populate data */
 			ret = read(pipe_rd[0], &msg_rd, sizeof(msg_rd));
+			if (ret < 0) {
+				if ((errno == EAGAIN) || (errno == EINTR))
+					goto redo_rd1;
+				pr_failed_dbg(name, "read");
+				break;
+			}
 			if (ret == 0)
 				break;
 			if (ret != sizeof(msg_rd)) {
@@ -185,7 +205,16 @@ cleanup:
 			int ret;
 
 			/* Wait for address of child's buffer */
+redo_rd2:
+			if (!opt_do_run)
+				break;
 			ret = read(pipe_wr[0], &msg_rd, sizeof(msg_rd));
+			if (ret < 0) {
+				if ((errno == EAGAIN) || (errno == EINTR))
+					goto redo_rd2;
+				pr_failed_dbg(name, "read");
+				break;
+			}
 			if (ret == 0)
 				break;
 			if (ret != sizeof(msg_rd)) {
@@ -234,9 +263,14 @@ cleanup:
 			msg_wr.val = val;
 			val++;
 
+redo_wr2:
+			if (!opt_do_run)
+				break;
 			/* Inform child that memory has been changed */
 			ret = write(pipe_rd[1], &msg_wr, sizeof(msg_wr));
 			if (ret < 0) {
+				if ((errno == EAGAIN) || (errno == EINTR))
+					goto redo_wr2;
 				if (errno != EBADF)
 					pr_failed_dbg(name, "write");
 				break;
